(****************************************************************************)
(*                                                                          *)
(*  Program language:  Borland Turbo Pascal  Version 6.0                    *)
(*  Program title: Calendar                                                 *)
(*                                                                          *)
(*  Written by T. R. Willmitch     1992 February 9                          *)
(*                                                                          *)
(*  Based on the Julian Day number to Gregorian calendar date algorithm     *)
(*  described on pages 63-64 of ASTRONOMICAL ALGORITHMS by Jean Meeus       *)
(*  (Richmond, 1991).                                                       *)
(*                                                                          *)
(*  Program summary:  Given the decimal Julian Day number (JD), this        *)
(*                    program determines the Gregorian calendar date and    *)
(*                    Universal Time.                                       *)
(*                                                                          *)
(*                    N.B., Although this program will yield negative years *)
(*                    it will not accept negative Julian Day numbers.       *)
(*                                                                          *)
(****************************************************************************)

PROGRAM Calendar;

USES Crt;

VAR  Year, Month, Day, Hour, Minute : INTEGER;
             JulianDay, DecimalDay, Second : DOUBLE;
                MonthName, Response : STRING;


PROCEDURE InputData;

VAR  Input : BOOLEAN;

BEGIN
  REPEAT
    Writeln('Enter the decimal Julian Day');
    Write('     number (DDDDDDD.DDDDDD):  ');
    Readln(JulianDay);

    Input := TRUE;

(* Do not accept the Julian Day number if it is negative.                   *)

    IF JulianDay < 0 THEN Input := FALSE;

  UNTIL Input = TRUE
END;  (* InputData *)


PROCEDURE CalDat(JD : DOUBLE;
                 VAR  Yr, Mo : INTEGER;
                 VAR    DecDay : DOUBLE);

(* Given the decimal Julian Day number (JD), the procedure CalDat           *)
(* determines the Gregorian calendar date (with decimal day).               *)

VAR Z, A, alpha, B, C, D, E : LONGINT;
                          F : DOUBLE;

BEGIN
  JD := JD + 0.5;
  Z := TRUNC(JD);
  F := JD - Z;

  IF Z < 2299161 THEN
    A := Z
  ELSE
    BEGIN
      alpha := TRUNC((Z - 1867216.25) / 36524.25);
      A := Z + 1 + alpha - alpha DIV 4
    END;  (* IF *)

    B := A + 1524;
    C := TRUNC((B - 122.1) / 365.25);
    D := TRUNC(365.25 * C);
    E := TRUNC((B - D) / 30.6001);

    DecDay := B - D - TRUNC(30.6001 * E) + F;

    IF E < 14 THEN
      Mo := E - 1
    ELSE
      Mo := E - 13;

    IF Mo > 2 THEN
      Yr := C - 4716
    ELSE
      Yr := C - 4715

END;  (* PROCEDURE CalDat *)


PROCEDURE DayTime(DecDay : DOUBLE;
                  VAR  Hr, Min : INTEGER;
                  VAR  Sec     : DOUBLE);

(* Given the decimal day, the procedure DayTime determines the time of day  *)
(* (hours, minutes, and seconds).                                           *)

VAR           Date : INTEGER;
     DecHr, DecMin : DOUBLE;

BEGIN
  Date := TRUNC(DecDay);
  DecHr := (DecDay - Date) * 24;
  Hr := TRUNC(DecHr);
  DecMin := (DecHr - Hr) * 60;
  Min := TRUNC(DecMin);
  Sec := (DecMin - Min) * 60
END;  (* PROCEDURE DayTime *)


BEGIN   (* Main program *)
  REPEAT
    ClrScr;
    GotoXY(23,7);
    Writeln('Julian Day Number to Calendar Date');
    Writeln;
    Writeln;
    InputData;
    CalDat(JulianDay, Year, Month, DecimalDay);
    CASE Month OF
       1 : MonthName := 'January';
       2 : MonthName := 'February';
       3 : MonthName := 'March';
       4 : MonthName := 'April';
       5 : MonthName := 'May';
       6 : MonthName := 'June';
       7 : MonthName := 'July';
       8 : MonthName := 'August';
       9 : MonthName := 'September';
      10 : MonthName := 'October';
      11 : MonthName := 'November';
      12 : MonthName := 'December';
    END;  (* CASE *)
    Day := TRUNC(DecimalDay);
    DayTime(DecimalDay, Hour, Minute, Second);
    Writeln;
    Writeln('         This corresponds to:  ',Year,' ',MonthName,' ',Day,'  ',
            Hour:2,':',Minute:2,':',Second:4:1,'  U.T.');
    Writeln; Writeln;
    Write('Determine another date (y/n)? ':54);
    Readln(Response);
  UNTIL (Response = 'n') OR (Response = 'N')
END.  (* Main program *)
