{$IFDEF SOLID}
unit Files;

interface

function SERVICE(ServiceNumber: Longint; Buffer: Pointer): Longint;

implementation
{$ELSE}
library Files;
{$ENDIF}

{$IFDEF VIRTUALPASCAL}
uses Types, Consts_, Log, Video, Wizard, Misc, Language, Config, Semaphor,
     Plugins;
{$IFNDEF SOLID}
{$Dynamic MAIN.LIB}
{$ENDIF}
{$ENDIF}
{$IFDEF DPMI}
uses
{$IFDEF SOLID}
     Plugins, Semaphor, Language, Misc, Config, Video,
{$ELSE}
     Decl,
{$ENDIF}
     Wizard, Consts_, Dos, Macroz, Types;
{$ENDIF}

const
 filesVersion = $00010300;
var
 Pool: TMemoryStream;
 FileName: String;

{$i files.inc}

function Startup: longint;
 var
  B: TBufStream;
 begin
  Startup:=srNo;

  FileName:=cGetParam('files.database');

  if Filename = '' then
   begin
    lngPrint('Main', 'files.wrong.database');
    sSetExitNow;
    Exit;
   end;

  Startup:=srYes;
  Filename:=FExpand(Filename);

  Pool.Init;
  B.Init(Filename, stOpenRead, 2048);

  if B.Status<>stOk then
   begin
    B.Done;

    lngBegin;
     lngPush(Filename);
     lngPrint('Main', 'files.database.created');
    lngEnd;

    Exit;
   end;

  Pool.CopyFrom(B, B.GetSize);
  B.Done;
 end;

procedure Shutdown;
 var
  B: TBufStream;
 begin
  repeat
   mCreate(JustPathName(FileName));

   B.Init(Filename, stCreate, 2048);

   if B.Status <> stOk then
    begin
     lngBegin;
     lngPush(Filename);
     lngPush(HexL(B.Status));
     lngPrint('Main', 'files.cannot.save');
     lngEnd;
     break;
    end;

   Pool.Seek(0);

   B.CopyFrom(Pool, Pool.GetSize);

  until True;

  B.Done;
  Pool.Done;
 end;

procedure Add;
 var
  Filename: String;
  Size: Longint;
  CRC: Longint;
 begin
  Filename:=StUpcase(sGetSemaphore('Files.Filename'));
  Size:=sGetNumSemaphore('Files.Size');
  CRC:=sGetNumSemaphore('Files.CRC');

  Pool.Seek(Pool.GetSize);
  Pool.WriteStr(Filename);
  Pool.Write(Size, SizeOf(Size));
  Pool.Write(CRC, SizeOf(CRC));

  lngBegin;
   lngPush(Filename);
   lngPush(Long2StrFmt(Size));
   lngPush(HexL(CRC));

   lngPrint('Main', 'files.added');
  lngEnd;
 end;

function Check: longint;
 var
  PoolSize: Longint;
  AFilename, BFilename: String;
  ASize, BSize: Longint;
  Strategy, stCount, AstCount: Longint;
  ACRC, BCRC: Longint;
 begin
  Pool.Seek(0);
  PoolSize:=Pool.GetSize;

  AFilename:=sGetSemaphore('Files.Filename');
  StUpcaseEx(AFilename);
  ASize:=sGetNumSemaphore('Files.Size');
  ACRC:=sGetNumSemaphore('Files.CRC');
  Strategy:=sGetNumSemaphore('Files.Strategy');

  StCount:=0;
  if Strategy and fsgName <> 0 then inc(StCount);
  if Strategy and fsgCRC <> 0 then inc(StCount);
  if Strategy and fsgSize <> 0 then inc(StCount);

  Check:=srNo;

  if stCount = 0 then Exit;

  while Pool.GetPos <> PoolSize do
   begin
    BFilename:=Pool.ReadStr;
    StUpcaseEx(BFilename);

    Pool.Read(BSize, SizeOf(BSize));
    Pool.Read(BCRC, SizeOf(BCRC));

    AstCount:=0;
    if (Strategy and fsgName <> 0) and (AFilename = BFilename) then Inc(AstCount);
    if (Strategy and fsgCRC <> 0) and (ACRC = BCRC) then Inc(AstCount);
    if (Strategy and fsgSize <> 0) and (ASize = BSize) then Inc(AstCount);

    if Astcount >= stCount then
     begin
      Check:=srYes;
      Exit;
     end;
   end;
 end;

function SERVICE(ServiceNumber: Longint; Buffer: Pointer): Longint; {$IFNDEF SOLID}export;{$ENDIF}
 begin
  Service:=srYes;
  case ServiceNumber of
   snStartup: Service:=Startup;
   snShutdown: Shutdown;
   snQueryName: sSetSemaphore('Kernel.Plugins.Info.Name','FILES');
   snQueryAuthor: sSetSemaphore('Kernel.Plugins.Info.Author','sergey korowkin');
   snQueryVersion: Service:=filesVersion;
   snQueryReqVer: Service:=kernelVersion;
   snfilCheck: Service:=Check;
   snfilAdd: Add;
  else
   Service:=srNotSupported;
  end;
 end;

{$IFNDEF SOLID}
exports
 Service;

begin
{$ENDIF}
end.
